import os
from functools import partial

from generate_cmake.log import Log


def _reduce_srcs(sources, base_directory, prejoin="${PROJECT_SOURCE_DIR}"):
    return map(lambda o: os.path.join(prejoin, o.replace(base_directory + '/', "")), sources)


def create_lib(build_item, base_directory=""):
    bloated_srcs = map(partial(os.path.join, build_item['location']), build_item['srcs'])
    reduced_srcs = _reduce_srcs(bloated_srcs, base_directory)
    txt = "add_library({} {})".format(build_item['target'], " ".join(reduced_srcs))
    if len(build_item['deps']):
        txt += "\ntarget_link_libraries({} {})".format(build_item['target'], " ".join(build_item['deps']))
    return txt


def create_bin(build_item, base_directory=""):
    reduced_srcs = _reduce_srcs(build_item['srcs'], base_directory)

    if 'is_test' in build_item.get('flags', []):
        assert len(reduced_srcs) == 1, "A test only supports one source file"
        txt = 'add_test({target} {srcs} {libs})'.format(
            target=build_item['target'],
            srcs=reduced_srcs[0],
            libs=" ".join(build_item['deps']) if len(build_item['deps']) else '""'
        )
    else:
        txt = "add_executable({} {})".format(build_item['target'], " ".join(reduced_srcs))
        if len(build_item['deps']):
            txt += "\ntarget_link_libraries({} {})".format(build_item['target'], " ".join(build_item['deps']))
    return txt


def should_create_target(build_item):
    if 'CMakeLists.txt' in os.listdir(build_item['location']):
        Log.debug("Ignoring: {}".format(build_item['target']))
        return False
    else:
        return True


def write_cmake(text, base_directory):
    write_folder = os.path.join(base_directory, "tmp")
    if not os.path.exists(write_folder):
        os.makedirs(write_folder)

    write_path = os.path.join(write_folder, "CMakeLists.txt")
    with open(write_path, 'w') as cmake_out:
        cmake_out.write(text)


def build_cmakes(to_build, base_directory):
    actions = {
        'lib': create_lib,
        'binary': create_bin
    }

    from graph import dependency_sort
    write_order = dependency_sort(to_build)

    Log.success('\n\nGenerating cmake....')
    cmake_text = ""
    for write in write_order:
        if write in to_build:
            build_item = to_build[write]
            if not should_create_target(build_item):
                continue

            action = actions[build_item['kind']]
            new_text = action(build_item, base_directory=base_directory)
            cmake_text += new_text + '\n'
            Log.info(new_text)

    write_cmake(cmake_text, base_directory)



try:
    from colorama import init as colorama_init
    from colorama import Fore, Style
    colorama_init()

except(ImportError):
    print "Could not import colorama: Skipping coloration"

    class Dummy(object):
        def __getattr__(self, key):
            return ''

    Fore = Dummy()
    Style = Dummy()

from collections import OrderedDict


class Log(object):
    _verbosities = OrderedDict([
        ("debug", Fore.LIGHTBLUE_EX),
        ("info", Fore.WHITE),
        ("success", Fore.LIGHTGREEN_EX),
        ("warn", Fore.YELLOW),
        ("error", Fore.RED),
    ])
    _enabled = _verbosities.keys()

    @classmethod
    def set_enabled(cls, keys):
        for key in keys:
            assert key in cls._verbosities.keys(), "{} unknown".format(key)
        cls._enabled = keys

    @classmethod
    def set_verbosity(cls, level):
        assert level in cls._verbosities.keys(), "{} unknown".format(level)
        all_keys = cls._verbosities.keys()
        start = all_keys.index(level)
        cls.set_enabled(all_keys[start:])
        cls.debug("Enabling ", all_keys[start:])

    @classmethod
    def get_verbosities(cls):
        return cls._verbosities


def make_logger(verbosity_type, color):
    def new_logger(cls, *txt):
        if verbosity_type in cls._enabled:
            out_str = " ".join(map(str, txt))
            print("{}{}{}".format(color, out_str, Style.RESET_ALL))

    return new_logger


for verbosity, color in Log._verbosities.items():
    setattr(Log, verbosity, classmethod(make_logger(verbosity, color)))


if __name__ == '__main__':
    Log.error("error:", 0)
    Log.warn("warn:", 1)
    Log.success("success:", 2)
    Log.info("info:", 3)
    Log.debug("debug:", 4)



from copy import deepcopy


def maybe_insert_deps(adj, key, data):
    if key in adj:
        if 'visited' in adj[key]:
            return
        deps = adj[key]['deps']
        adj[key]['visited'] = True

        for dep in deps:
            maybe_insert_deps(adj, dep, data)

    else:
        adj[key] = {'visited': True}
    data.append(key)


def dependency_sort(adj):
    our_adj = deepcopy(adj)
    assert len(our_adj.keys()) > 0, "Must have a graph!"
    result = []
    for element in adj:
        maybe_insert_deps(our_adj, element, result)
    return result

if __name__ == '__main__':
    test = {
        'a': {'deps': ['b', 'd']},
        'b': {'deps': ['c']},
        'd': {'deps': ['c']},
        'e': {'deps': ['d']},
    }

    print dependency_sort(test)



"""cppmake auto-generates a big CMake file from C++ headers
"""
import setuptools

description = "Python Super CMake"

setuptools.setup(
    name='generate_cmake',
    version='1.1.1',
    license='MIT',
    long_description=__doc__,
    url='https://github.com/jpanikulam/experiments/tree/master/pymake',
    author='Jacob Panikulam',
    author_email='jpanikul@gmail.com',
    packages=setuptools.find_packages(),
    description=description,
    keywords="cmake make stupid",
    platforms='any',
    zip_safe=True,
    scripts=['pymake'],
    install_requires=[
        'colorama==0.3.9',
    ],

)
